@@ -471,6 +471,8 @@ swi_message(determinism_error(Goal, det, nondet, goal)) -->
     [ 'Goal ~p succeeded with a choice point'-[Goal] ].
 swi_message(qlf_format_error(File, Message)) -->
     [ '~w: Invalid QLF file: ~w'-[File, Message] ].
+swi_message(goal_expansion_error(bound, Term)) -->
+    [ 'Goal expansion bound a variable to ~p'-[Term] ].
 
 det_error(nondet) -->
     [ ' succeeded with a choicepoint'- [] ].
@@ -1185,6 +1187,12 @@ prolog_message(close_on_abort(Stream)) -->
     [ 'Abort: closed stream ~p'-[Stream] ].
 prolog_message(cancel_halt(Reason)) -->
     [ 'Halt cancelled: ~p'-[Reason] ].
+prolog_message(on_error(halt(Status))) -->
+    { statistics(errors, Errors),
+      statistics(warnings, Warnings)
+    },
+    [ 'Halting with status ~w due to ~D errors and ~D warnings'-
+      [Status, Errors, Warnings] ].
 
 prolog_message(query(QueryResult)) -->
     query_result(QueryResult).
@@ -1761,12 +1769,25 @@ print_message_guarded(Level, Term) :-
                 ;   notrace(user:message_hook(Term, Level, Lines))
                 )
             ->  true
-            ;   print_system_message(Term, Level, Lines)
+            ;   '$inc_message_count'(Level),
+                print_system_message(Term, Level, Lines),
+                maybe_halt_on_error(Level)
             )
         )
     ;   true
     ).
 
+maybe_halt_on_error(error) :-
+    current_prolog_flag(on_error, halt),
+    !,
+    halt(1).
+maybe_halt_on_error(warning) :-
+    current_prolog_flag(on_warning, halt),
+    !,
+    halt(1).
+maybe_halt_on_error(_).
+
+
 %!  print_system_message(+Term, +Kind, +Lines)
 %
 %   Print the message if the user did not intecept the message.
