@@ -152,7 +152,7 @@ user:portray(ATrie) :-
     ).
 user:portray(Cont) :-
     compound(Cont),
-    compound_name_arguments(Cont, '$cont$', [Clause, PC | Args]),
+    compound_name_arguments(Cont, '$cont$', [_Context, Clause, PC | Args]),
     clause_property(Clause, file(File)),
     file_base_name(File, Base),
     clause_property(Clause, line_count(Line)),
@@ -1956,7 +1956,11 @@ false_path(ATrie, [ATrie|T], Seen) :-
     '$idg_false_edge'(ATrie, Dep, Status),
     tdebug(reeval, '    ~p has dependent ~p (~w)', [ATrie, Dep, Status]),
     (   Status == invalid
-    ->  false_path(Dep, T, [ATrie|Seen])
+    ->  (   false_path(Dep, T, [ATrie|Seen])
+        ->  true
+        ;   length(Seen, Len),               % invalid has no dependencies:
+            T = [s(2, Len, [])]              % dynamic and tabled or explicitly
+        )                                    % invalidated
     ;   status_rank(Status, Rank),
         length(Seen, Len),
         T = [s(Rank,Len,Dep)]
@@ -2024,6 +2028,7 @@ reeval_monotonic_node(ATrie, Size) :-
                 '$clause'(Head, _Body, ClauseRef, _Bindings),
                 tdebug(monotonic, 'Propagating ~p from ~p to ~p',
                        [Head, _0SrcTrie, ATrie]),
+                '$idg_set_current'(_, ATrie),
                 pdelim(Cont, Skel, ATrie),
                 fail
             ;   '$idg_mono_empty_queue'(DepRef, Count)
@@ -2033,6 +2038,7 @@ reeval_monotonic_node(ATrie, Size) :-
                 '$tbl_node_answer'(Node, SrcSkel),
                 tdebug(monotonic, 'Propagating ~p from ~p to ~p',
                        [Skel, _0SrcTrie, ATrie]),
+                '$idg_set_current'(_, ATrie),
                 pdelim(Cont, Skel, ATrie),
                 fail
             ;   '$idg_mono_empty_queue'(DepRef, Count)
