@@ -142,8 +142,8 @@ setupProlog(void)
   initFlags();
   DEBUG(1, Sdprintf("Foreign Predicates ...\n"));
   initBuildIns();
-  DEBUG(1, Sdprintf("TCMalloc binding ...\n"));
-  initTCMalloc();
+  DEBUG(1, Sdprintf("Malloc binding ...\n"));
+  initMalloc();
   DEBUG(1, Sdprintf("Operators ...\n"));
   initOperators();
   DEBUG(1, Sdprintf("GMP ...\n"));
@@ -1345,9 +1345,9 @@ PRED_IMPL("$on_signal", 4, on_signal, 0)
       succeed;
     }
 
-    return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_signal_handler, sig);
+    return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_signal_handler, new);
   } else
-    return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_signal_handler, sig);
+    return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_signal_handler, new);
 
   succeed;
 }
