@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2020, University of Amsterdam
+    Copyright (c)  1985-2021, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -255,6 +256,10 @@ static bool		hasAlternativesFrame(LocalFrame);
 static void		alternatives(Choice);
 static int		exceptionDetails(void);
 static int		listGoal(LocalFrame frame);
+static LocalFrame	frameAtLevel(LocalFrame frame, int at_depth,
+				     bool interactive);
+static int		saveGoal(LocalFrame frame, int at_depth,
+				 bool interactive);
 static int		traceInterception(LocalFrame, Choice, int, Code);
 static int		traceAction(char *cmd,
 				    int port,
@@ -506,7 +511,7 @@ again:
       }
       buf[0] = c;
       buf[1] = EOS;
-      if ( isDigit(buf[0]) || buf[0] == '/' )
+      if ( isDigit(buf[0]) || buf[0] == '/' || buf[0] == '-' )
       { Sfprintf(Sdout, buf);
 	readLine(Sdin, Sdout, buf);
       }
@@ -636,6 +641,7 @@ traceAction(char *cmd, int port, LocalFrame frame, Choice bfr,
 	    bool interactive)
 { GET_LD
   int num_arg;				/* numeric argument */
+  int def_arg = TRUE;			/* arg is default */
   char *s;
 
 #define FeedBack(msg)	{ if (interactive) { if (cmd[1] != EOS) \
@@ -647,17 +653,17 @@ traceAction(char *cmd, int port, LocalFrame frame, Choice bfr,
 			  else \
 			    warning(msg); \
 			}
-#define Default		(-1)
 
   for(s=cmd; *s && isBlank(*s); s++)
     ;
-  if ( isDigit(*s) )
+  if ( isDigit(*s) || (*s == '-' && s[1] && isDigit(s[1])) )
   { num_arg = strtol(s, &s, 10);
 
     while(isBlank(*s))
       s++;
+    def_arg = FALSE;
   } else
-    num_arg = Default;
+    num_arg = 0;
 
   switch( *s )
   { case 'a':	FeedBack("abort\n");
@@ -700,10 +706,20 @@ traceAction(char *cmd, int port, LocalFrame frame, Choice bfr,
 		} else
 		  Warn("Can't ignore goal at this port\n");
 		return ACTION_CONTINUE;
-    case 'r':	if (port & (REDO_PORT|FAIL_PORT|EXIT_PORT|EXCEPTION_PORT))
-		{ FeedBack("retry\n[retry]\n");
-		  debugstatus.retryFrame = consTermRef(frame);
-		  return ACTION_RETRY;
+    case 'r':	if ( !def_arg ||
+		     (port & (REDO_PORT|FAIL_PORT|EXIT_PORT|EXCEPTION_PORT)) )
+		{ LocalFrame fr;
+
+		  if ( (fr = frameAtLevel(frame, def_arg ? 0 : num_arg,
+					  interactive)) )
+		  { if ( interactive )
+		      Sfprintf(Sdout, "retry\nretry %s at level %d\n",
+			       predicateName(fr->predicate), levelFrame(fr));
+		    debugstatus.retryFrame = consTermRef(fr);
+		    return ACTION_RETRY;
+		  } else
+		  { return ACTION_CONTINUE;
+		  }
 		} else
 		  Warn("Can't retry at this port\n");
 		return ACTION_CONTINUE;
@@ -719,7 +735,7 @@ traceAction(char *cmd, int port, LocalFrame frame, Choice bfr,
 		debugstatus.skiplevel = levelFrame(frame) - 1;
 		return ACTION_CONTINUE;
     case 'd':   FeedBack("depth\n");
-                setPrintOptions(consInt(num_arg));
+                setPrintOptions(def_arg ? 10 : consInt(num_arg));
 		return ACTION_AGAIN;
     case 'w':   FeedBack("write\n");
                 setPrintOptions(ATOM_write);
@@ -735,7 +751,7 @@ traceAction(char *cmd, int port, LocalFrame frame, Choice bfr,
 		debugmode(DBG_OFF, NULL);
 		return ACTION_CONTINUE;
     case 'g':	FeedBack("goals\n");
-		PL_backtrace(num_arg == Default ? 5 : num_arg, PL_BT_USER);
+		PL_backtrace(def_arg ? 5 : num_arg, PL_BT_USER);
 		return ACTION_AGAIN;
     case 'A':	FeedBack("alternatives\n");
 		alternatives(bfr);
@@ -756,6 +772,9 @@ traceAction(char *cmd, int port, LocalFrame frame, Choice bfr,
     case 'L':	FeedBack("Listing");
 		listGoal(frame);
 		return ACTION_AGAIN;
+    case 'S':	FeedBack("Save goal");
+		saveGoal(frame, def_arg ? 0 : num_arg, interactive);
+		return ACTION_AGAIN;
     case '+':	FeedBack("spy\n");
 		set(frame->predicate, SPY_ME);
 		return ACTION_AGAIN;
@@ -765,37 +784,87 @@ traceAction(char *cmd, int port, LocalFrame frame, Choice bfr,
     case '?':
     case 'h':	helpTrace();
 		return ACTION_AGAIN;
-    case 'D':   GD->debug_level = num_arg;
+#ifdef O_DEBUG
+    case 'D':   GD->debug_level = def_arg ? 0 : num_arg;
 		FeedBack("Debug level\n");
 		return ACTION_AGAIN;
+#endif
     default:	Warn("Unknown option (h for help)\n");
 		return ACTION_AGAIN;
   }
 }
 
+typedef struct trace_command
+{ const char *keys;
+  const char *comment;
+} trace_command;
+
+static void
+set_max_len(int *i, const char *s)
+{ size_t len = strlen(s);
+  if ( len > (int)*i )
+    *i = (int)len;
+}
+
 static void
 helpTrace(void)
 { GET_LD
+  const trace_command commands[] =
+  {
+    { "+",		   "set spy point" },
+    { "-",		   "clear spy point" },
+    { "/c|e|r|f|u|a goal", "find goal at port" },
+    { ".",		   "repeat find" },
+    { "a",		   "abort to toplevel" },
+    { "A",		   "alternatives" },
+    { "b",		   "break (new toplevel)" },
+    { "c (ret, space)",	   "creep to next port" },
+    { "[depth] d",	   "depth for printing" },
+    { "e",		   "exit Prolog" },
+    { "f",		   "make goal fail" },
+    { "[depth] g",	   "backtrace (-N from top)" },
+    { "h (?)",		   "help" },
+    { "i",		   "ignore current goal" },
+    { "l",		   "leap to spy point" },
+    { "L",		   "list current goal" },
+    { "[level] r",	   "retry goal [at level]" },
+    { "s",		   "skip over" },
+    { "[level] S",	   "save goal [at level]" },
+    { "u",		   "up (complete goal)" },
+    { "s",		   "(quoted) write goals" },
+    { "m",		   "exception details" },
+    { "C",		   "toggle show context" },
+  #if O_DEBUG
+    { "[level] D",         "system debug level" },
+  #endif
+    { NULL,		   NULL }
+  };
+  const trace_command *cmd;
+  int lkw = 0, lcw = 0, rkw = 0, rcw = 0;
 
-  Sfprintf(Sdout,
-	   "Options:\n"
-	   "+:                  spy        -:              no spy\n"
-	   "/c|e|r|f|u|a goal:  find       .:              repeat find\n"
-	   "a:                  abort      A:              alternatives\n"
-	   "b:                  break      c (ret, space): creep\n"
-	   "[depth] d:          depth      e:              exit\n"
-	   "f:                  fail       [ndepth] g:     goals (backtrace)\n"
-	   "h (?):              help       i:              ignore\n"
-	   "l:                  leap       L:              listing\n"
-	   "n:                  no debug   p:              print\n"
-	   "r:                  retry      s:              skip\n"
-	   "u:                  up         w:              write\n"
-	   "m:                  exception details\n"
-	   "C:                  toggle show context\n"
-#if O_DEBUG
-	   "[level] D:	      set system debug level\n"
-#endif
-	   "");
+  for(cmd=commands; cmd->keys; cmd+=2)
+  { set_max_len(&lkw, cmd[0].keys);
+    set_max_len(&lcw, cmd[0].comment);
+
+    if ( cmd[1].keys )
+    { set_max_len(&rkw, cmd[1].keys);
+      set_max_len(&rcw, cmd[1].comment);
+    } else
+      break;
+  }
+
+  Sfprintf(Sdout, "Trace commands:\n");
+  for(cmd=commands; cmd->keys; cmd+=2)
+  { if ( cmd[1].keys )
+    { Sfprintf(Sdout, "%-*s %-*s | %-*s %s\n",
+	       lkw, cmd[0].keys, lcw, cmd[0].comment,
+	       rkw, cmd[1].keys,      cmd[1].comment);
+    } else
+    { Sfprintf(Sdout, "%-*s %s\n",
+	       lkw, cmd[0].keys, cmd[0].comment);
+      break;
+    }
+  }
 }
 
 
@@ -1074,6 +1143,54 @@ listGoal(LocalFrame frame)
 }
 
 
+static LocalFrame
+frameAtLevel(LocalFrame frame, int at_depth, bool interactive)
+{ GET_LD
+
+  if ( at_depth )
+  { pl_context_t ctx;
+
+    if ( PL_get_context(&ctx, 0) )
+    { do
+      { if ( levelFrame(ctx.fr) == at_depth )
+	  return ctx.fr;
+      } while(PL_step_context(&ctx));
+
+      if ( interactive )
+	Sfprintf(Sdout, "No frame at level %d", at_depth);
+
+      return NULL;
+    }
+
+    return NULL;
+  }
+
+  return frame;
+}
+
+
+static int
+saveGoal(LocalFrame frame, int at_depth, bool interactive)
+{ GET_LD
+  term_t goal;
+
+  if ( !(frame = frameAtLevel(frame, at_depth, interactive)) )
+    return FALSE;
+
+  if ( (goal = PL_new_term_ref()) &&
+       put_frame_goal(goal, frame) &&
+       PL_record_az(ATOM_saved_goals, goal, 0, RECORDA) )
+  { if ( interactive )
+      Sfprintf(Sdout, "\nRecorded goal to key `saved_goals`\n");
+    return TRUE;
+  }
+
+  Sfprintf(Sdout, "\nFailed to save goal\n");
+  return FALSE;
+}
+
+
+
 static void
 writeContextFrame(IOSTREAM *out, pl_context_t *ctx, int flags)
 { if ( (flags&PL_BT_SAFE) )
@@ -1087,6 +1204,8 @@ writeContextFrame(IOSTREAM *out, pl_context_t *ctx, int flags)
 }
 
 
+#define SHOW_FRAME(fr) ( isDebugFrame(fr) || !(flags&PL_BT_USER) )
+
 static void
 _PL_backtrace(IOSTREAM *out, int depth, int flags)
 { pl_context_t ctx;
@@ -1104,6 +1223,26 @@ _PL_backtrace(IOSTREAM *out, int depth, int flags)
     if ( SYSTEM_MODE )
       flags &= ~PL_BT_USER;
 
+    if ( depth < 0 )			/* deph < 0: top depth frames */
+    { pl_context_t from = ctx;
+      int skip;
+
+      skip = depth = -depth;
+      while( PL_step_context(&ctx) )
+      { if ( SHOW_FRAME(ctx.fr) && --skip <= 0 )
+	  break;
+      }
+      while( PL_step_context(&ctx) )
+      { if ( SHOW_FRAME(ctx.fr) )
+	{ do
+	  { PL_step_context(&from);
+	  } while( !SHOW_FRAME(from.fr) );
+	}
+      }
+
+      ctx = from;
+    }
+
     for(; depth > 0; PL_step_context(&ctx))
     { LocalFrame frame;
 
@@ -1128,7 +1267,7 @@ _PL_backtrace(IOSTREAM *out, int depth, int flags)
 	def = frame->predicate;
       }
 
-      if ( isDebugFrame(frame) || !(flags&PL_BT_USER) )
+      if ( SHOW_FRAME(frame) )
       { writeContextFrame(out, &ctx, flags);
 	depth--;
       }
