@@ -36,7 +36,10 @@
 */
 
 /*#define O_DEBUG 1*/
-#include "pl-incl.h"
+#define USE_FLI_INLINES 1
+#define USE_ALLOC_INLINES 1
+
+#include "pl-wam.h"
 #include "pl-comp.h"
 #include "pl-arith.h"
 #include "pl-inline.h"
@@ -46,6 +49,21 @@
 #include "pl-event.h"
 #include "pl-tabling.h"
 #include "pl-undo.h"
+#include "pl-util.h"
+#include "pl-gc.h"
+#include "pl-trace.h"
+#include "pl-pro.h"
+#include "pl-modul.h"
+#include "pl-funct.h"
+#include "pl-fli.h"
+#include "pl-proc.h"
+#include "pl-attvar.h"
+#include "pl-setup.h"
+#include "pl-prims.h"
+#include "pl-write.h"
+#include "pl-supervisor.h"
+#include "pl-index.h"
+#include "pl-cont.h"
 #include <fenv.h>
 #ifdef _MSC_VER
 #pragma warning(disable: 4102)		/* unreferenced labels */
@@ -231,10 +249,6 @@ DbgPrintInstruction(LocalFrame FR, Code PC)
 
 
 
-#include "pl-alloc.c"
-#include "pl-index.c"
-
-
 		 /*******************************
 		 *	     SIGNALS		*
 		 *******************************/
@@ -1762,7 +1776,7 @@ TBD: In these modern days we can  probably   do  GC. Still, if it is not
 needed why would we?
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static void
+void
 resumeAfterException(int clear, Stack outofstack)
 { GET_LD
 
@@ -1810,8 +1824,6 @@ exceptionUnwindGC(void)
 		 *   FOREIGN-LANGUAGE INTERFACE *
 		 *******************************/
 
-#include "pl-fli.c"
-
 #ifdef O_DEBUGGER
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 findStartChoice(LocalFrame fr, Choice ch)
